import re
from bloonix.heaven import H
from bloonix.heaven.plugin.crud import Auto, Show, Options, Delete, Validate, RaiseOnDupError
from bloonix.runcode import RunCode
from bloonix.netaddr import get_ip_by_hostname
from bloonix import rx
from bloonix.exception import BloonixDuplicationError

class Controller(Auto, Show, Options, Delete, Validate, RaiseOnDupError):
    def startup(self):
        self.db = H.model.db.host
        H.route("/config/hosts/except/:except_host_id", "except_host_id")
        H.route("/config/hosts/dnsreq", "dnsreq")
        H.route("/config/hosts", "list")
        H.route("/config/hosts/:id", "show")
        H.route("/config/hosts/:id/update", "update")
        H.route("/config/hosts/:id/delete", "delete")
        H.route("/config/hosts/:id/options", "options")
        H.route("/config/hosts/:id/variables", "variables")
        H.route("/config/hosts/options", "options")
        H.route("/config/hosts/create", "create")

    def except_host_id(self, opts):
        ret = H.model.db.host.query(
            query=H.req.get_query_data(),
            count="id",
            table="host",
            columns=["id", "hostname"],
            condition=["where", ["id", "!=", opts["except_host_id"]]],
            order=["asc", "host.hostname"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def dnsreq(self, opts):
        RunCode(5, self._dnsreq)

    def _dnsreq(self):
        hostname = H.req.param("hostname")
        if re.search(rx.HOSTNAME, hostname):
            try:
                H.stash.data["v4"] = get_ip_by_hostname(hostname, "v4")
                H.stash.data["v6"] = get_ip_by_hostname(hostname, "v6")
            except: pass

    def list(self, opts):
        ret = H.model.db.host.query(
            query=H.req.get_query_data(),
            count="id",
            table="host",
            parse_int=[
                "id",
                "enabled",
                "notification_interval",
                "notification_enabled",
            ],
            columns=[
                "id",
                "hostname",
                "ipaddr1",
                "ipaddr2",
                "ipaddr3",
                "ipaddr4",
                "description",
                "comment",
                "project_class",
                "location_class",
                "enabled",
                "notification_interval",
                "notification_enabled",
                "country_code"
            ],
            order=["asc", "host.hostname"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        for row in ret.data:
            row["templates"] = H.model.db.template.by_host_id(row["id"])

        H.stash.data = ret.data
        H.stash.total = ret.total

    def options(self, opts):
        if "id" in opts:
            H.model.db.host.init("update")
        else:
            H.model.db.host.init("create")

        super().options(opts)

        if "id" in opts:
            host_secret = H.model.db.host_secret.find("host_id", opts["id"])
            H.stash.data["values"]["password"] = host_secret["password"]

    def create(self, opts):
        self.db.init("create")

        data = self.validate(opts)
        if not data:
            return

        password = data["password"]
        del data["password"]

        group_data = {}
        for key in ["group_id", "location_group_id", "contactgroup_id", "template_id"]:
            if key in data:
                group_data[key] = data[key]
                del data[key]
            else:
                group_data[key] = []

        try:
            ret = self.db.create_and_get(data)
        except BloonixDuplicationError:
            return self.raise_on_dup_error()
        except Exception:
            raise
        H.model.db.host_secret.create({
            "host_id": ret["id"],
            "password": password
        })

        for group_id in group_data["group_id"]:
            try:
                H.model.db.host_group.create({
                    "host_id": ret["id"],
                    "group_id": group_id
                })
            except Exception: pass

        for location_group_id in group_data["location_group_id"]:
            try:
                H.model.db.host_location_group.create({
                    "host_id": ret["id"],
                    "location_group_id": location_group_id
                })
            except Exception: pass

        for contactgroup_id in group_data["contactgroup_id"]:
            try:
                H.model.db.host_contactgroup.create({
                    "host_id": ret["id"],
                    "contactgroup_id": contactgroup_id
                })
            except Exception: pass

        for template_id in group_data["template_id"]:
            try: H.model.db.host_template.add_host(template_id, ret["id"])
            except Exception: pass

        ret["password"] = password
        sys_options = H.model.db.sys_options.get_keys("server_names")
        ret["bloonix_server_names"] = sys_options.get(
            "server_names",
            H.config["bloonix_server_names"])
        if len(H.config["bloonix_proxy_id"]):
            ret["bloonix_proxy_id"] = H.config["bloonix_proxy_id"]
        H.stash.data = ret

    def update(self, opts):
        self.db.init("update")

        data = self.validate(opts, ignore_missing=True)
        if not data:
            return

        if "password" in data:
            H.model.db.host_secret.update_by_col("host_id", opts["id"], {
                "password": data["password"]
            })
            del data["password"]

        try:
            self.db.update_by_id(opts["id"], data)
        except BloonixDuplicationError:
            return self.raise_on_dup_error()
        except Exception:
            raise

    def variables(self, opts):
        H.stash.data = H.model.db.host.get_all_variables(opts["id"])
