from bloonix.heaven import H
from bloonix.heaven.validator import Validator
import bloonix.rx as rx
import bloonix.utils as utils
import math
import re
import time

class Controller:
    def startup(self):
        # The check :id + :service_id is done in __init__.py
        H.route("/hosts/charts", "charts")
        H.route("/hosts/:id/charts", "charts")
        H.route("/hosts/:id/services/:service_id/charts/:chart/data", "charts_data")
        H.route("/hosts/:id/services/:service_id/charts/data", "charts_data_raw")

    def charts(self, opts):
        ret = H.model.db.chart.list(H.req.get_query_data(), opts.get("id", None))

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def charts_data(self, opts):
        v = Validator()
        v.set({
            "last": {
                "type": int,
                "maxvalue": 34560000, # 400 days
                "default": 3600,
            },
            "from_time": {
                "type": str,
                "regex": rx.DATETIME,
                "optional": True,
            },
            "to_time": {
                "type": str,
                "regex": rx.DATETIME,
                "optional": True,
            },
            "type": {
                "type": str,
                "options": ["min", "max", "avg"],
                "default": "avg",
            },
            "satellite_id": {
                "type": list,
                "multioptions": H.model.db.satellite.get_ids(),
                "allow_empty": True,
                "default": []
            },
            # the size in pixel of the chart container
            "datapoints": {
                "type": int,
                "default": 0,
            }
        })

        res = v.validate(H.req.data)
        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        service = H.model.db.service.by_id_with_hostname(opts["service_id"])
        if not service:
            return H.plugin.error.set("err-600")

        plugin = H.model.db.plugin.find(
            "command", service["command"])
        if not plugin:
            return H.plugin.error.set("err-600")

        chart = H.model.db.chart.find(
            "command", service["command"],
            "chart_id", opts["chart"])
        if not chart:
            return H.plugin.error.set("err-600")

        item_ids = []
        keys_by_item_id = {}

        for s in chart["series"]:
            if "item_id" not in s:
                raise Exception(
                    "missing item_id for item '%s' command '%s'"
                    % (s["name"], plugin["command"]))
            keys_by_item_id[s["item_id"]] = s
            item_ids.append(s["item_id"])

        to_time = int(time.time())
        if res.data["from_time"]:
            from_time = utils.unixtime(
                res.data["from_time"],
                H.user["timezone"])
            if res.data["to_time"] and res.data["to_time"] > res.data["from_time"]:
                to_time = utils.unixtime(
                    res.data["to_time"],
                    H.user["timezone"])
        else:
            from_time = int(time.time()) - res.data["last"]

        calculated_items = (to_time - from_time) / service["interval"]
        if calculated_items > 20160:
            H.log.info(
                "get chart data for service",
                service["id"],
                "calculated items =",
                calculated_items,
                "switch to stat_hourly,",
                "column name", res.data["type"])
            model = H.model.ds.stat_hourly
            column = res.data["type"]
        else:
            model = H.model.ds.stat
            column = "value"

        #model = H.model.ds.stat
        #column = "value"

        datapoints = res.data["datapoints"]
        data = {}
        log_time = [""]
        total_time = time.time()
        for item_id in item_ids:
            log_time.append("    - item_id = %s" % item_id)
            start_time = time.time()
            rows = model.data_by_item_id(
                service_id=service["id"],
                item_id=item_id,
                from_time=from_time,
                to_time=to_time,
                satellite_id=res.data["satellite_id"])

            log_time.append("      rows = %s" % len(rows))
            log_time.append("      select = %.3fs" % (time.time() - start_time))
            start_time = time.time()

            opposite = keys_by_item_id[item_id].get("opposite", False)
            item_id = keys_by_item_id[item_id]["name"]
            data[item_id] = []
            _type = res.data["type"]

            if datapoints == 0 or len(rows) <= datapoints:
                for row in rows:
                    v = row[column]
                    if opposite is True:
                        v *= -1
                    data[item_id].append([row["time"] * 1000, v])
            elif _type == "avg":
                i = 0
                n = 0
                m = math.ceil(len(rows) / datapoints)
                for row in rows:
                    i += 1
                    n += row[column]
                    if i == m:
                        v = "%.3f" % (n / m)
                        if opposite is True:
                            v *= -1
                        data[item_id].append([row["time"] * 1000, v])
                        i = 0
                        n = 0
            elif res.data["type"] == "min" or res.data["type"] == "max":
                i = 0
                n = 0
                m = math.ceil(len(rows) / datapoints)
                for row in rows:
                    i += 1
                    if ((i == 1)
                            or (_type == "min" and n > row[column])
                            or (_type == "max" and n < row[column])):
                        n = row[column]
                    if i == m:
                        i = 0
                        if opposite is True:
                            n *= -1
                        data[item_id].append([
                            row["time"] * 1000,
                            n])

            log_time.append("      processing = %.3fs" % (time.time() - start_time))

        if H.config["log_stat_times"]:
            log_time[0] = "get stat cmd %s %s %.3fs" % (
                plugin["command"],
                res.data["type"],
                time.time() - total_time)
            H.log.notice("\n".join(log_time))

        H.stash.data = {
            "service": service,
            "plugin": plugin,
            "chart": chart,
            "data": data,
        }

    def charts_data_raw(self, opts):
        v = Validator()
        v.set({
            "last": {
                "type": int,
                "maxvalue": 604800, # 7 days
                "default": 86400,
            },
            "from_time": {
                "type": str,
                "regex": rx.DATE,
                "optional": True,
            },
            "to_time": {
                "type": str,
                "regex": rx.DATE,
                "optional": True,
            },
            "satellite_id": {
                "type": list,
                "multioptions": H.model.db.satellite.get_ids(),
                "allow_empty": True,
                "default": []
            },
            "offset": {
                "type": int,
                "default": 0,
            },
            "limit": {
                "type": int,
                "default": 50,
                "maxvalue": 1000,
            }
        })

        res = v.validate(H.req.data)
        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        from_time = res.data["from_time"]
        to_time = res.data["to_time"]
        if from_time and to_time:
            from_time = utils.unixtime(
                "%s 00:00:00" % from_time,
                H.user["timezone"])
            to_time = utils.unixtime(
                "%s 23:59:59" % to_time,
                H.user["timezone"])
        else:
            to_time = int(time.time())
            from_time = to_time - res.data["last"]

        if to_time < from_time:
            return H.plugin.error.set("err-622")
        if to_time - from_time > 86400 * 400:
            return H.plugin.error.set("err-803")

        items = H.model.ds.stat.data(
            service_id=H.cache["service"]["id"],
            from_time=from_time,
            to_time=to_time,
            satellite_id=res.data["satellite_id"],
            with_satellite=True,
            order="desc")

        service = H.cache["service"]
        plugin = H.model.db.plugin.by_command(service["command"])
        metrics_by_item_id = {}
        for name, opts in plugin["metrics"].items():
            metrics_by_item_id[opts["item_id"]] = name
        rows = []
        row = None
        while len(items):
            item = items.pop(0)
            item_id = item["item_id"]
            if item_id not in metrics_by_item_id:
                continue
            key = metrics_by_item_id[item_id]
            if row == None or key in row:
                if row != None:
                    rows.append(row)
                row = {
                    "time": item["time"],
                    "satellite_id": item["satellite_id"]}
            row[key] = item["value"]
        if row != None and len(row):
            rows.append(row)

        offset = res.data["offset"]
        limit = res.data["limit"] + res.data["offset"]
        H.stash.data = rows[offset:limit]
        H.stash.total = len(rows)
