import re
from bloonix.heaven import H
from bloonix.dbi.crud import CRUD
from bloonix.heaven.validator import Validator
from bloonix import ccodes
from bloonix import rx
from bloonix.json import json

class MessageService(CRUD):
    def __init__(self):
        self.validator = {
            "sendmail": self._init_sendmail_params(),
            "http": self._init_http_params(),
            "script": self._init_script_params()
        }

    def _default_params(self):
        return {
            "name": {
                "type": str,
                "minlength": 1,
                "maxlength": 30
            },
            "enabled": {
                "type": int,
                "options": [0,1],
                "default": 1
            }
        }

    def _init_sendmail_params(self):
        self._v_sendmail = Validator()
        self._v_sendmail.set(self._default_params())
        self._v_sendmail.set({
            "message_template": {
                "type": str,
                "default": "\n".join([
                    "*** Status for host %hostname% (%ipaddr%) at %date% ***",
                    "",
                    "https://%webgui_domain%/#hosts/%host_id%",
                    "%host_description%",
                    "",
                    "%begin_loop%",
                    "---",
                    "Service ID: %service_id%",
                    "Service: %service_name%",
                    "Status: %service_status%",
                    "Message: %service_message%",
                    "%end_loop%"
                ]),
                "minlength": 0,
                "maxlength": 20000
            },
            "sender": {
                "type": str,
                "default": "bloonix@localhost",
                "maxlength": 50
            },
            "send_to": {
                "type": str,
                "default": "%send_to%",
                "maxlength": 100
            },
            "subject": {
                "type": str,
                "default": "Bloonix-Monitoring Status Message",
                "maxlength": 255
            },
            "message": {
                "type": str,
                "default": "%message%",
                "maxlength": 100
            }
        })
        return self._v_sendmail

    def _validate_data_type_json(self, data):
        try:
            if "postdata_json" in data and data["postdata_json"] != None:
                json.loads(data["postdata_json"])
        except Exception as err:
            return {"postdata_json": "err-720"}

        try:
            if "postdata_urlencoded" in data and data["postdata_urlencoded"] != None:
                for k,v in data["postdata_urlencoded"].items():
                    if not isinstance(k, str) or not isinstance(v, (str, list)):
                        return {"postdata_urlencoded": "err-720"}
                    if isinstance(v, list):
                        for i in v:
                            if not isinstance(i, str):
                                return {"postdata_urlencoded": "err-720"}
        except Exception:
            return {"postdata_urlencoded": "err-720"}

        try:
            if "header" in data and data["header"] != None:
                for k,v in data["header"].items():
                    if not isinstance(k, str) or not isinstance(v, str):
                        return {"postdata_urlencoded": "err-720"}
        except Exception:
            return {"header": "err-720"}

    def _init_http_params(self):
        self._v_http = Validator(
            no_mixin=[ ["postdata_urlencode", "postdata_json", "postdata_raw"] ],
            post_validation=self._validate_data_type_json)
        self._v_http.set(self._default_params())
        self._v_http.set({
            "message_template": {
                "type": str,
                "minlength": 0,
                "maxlength": 20000,
                "default": "%short_message%"
            },
            "url": {
                "type": str,
                "regex": rx.HTTP_URL,
                "maxlength": 1000
            },
            "insecure": {
                "type": int,
                "options": [0,1],
                "default": 0
            },
            "auth_basic": {
                "type": str,
                "minlength": 0,
                "maxlength": 100,
                "optional": True
            },
            "user_agent": {
                "type": str,
                "maxlength": 120,
                "default": "Bloonix-Monitoring-Message-Service"
            },
            "header": {
                "type": dict,
                "allow_empty": True,
                "maxlength": 1024,
                "default": {}
            },
            "postdata_urlencoded": {
                "type": dict,
                "allow_empty": True,
                "maxlength": 10000,
                "optional": True,
            },
            "postdata_json": {
                "type": str,
                "minlength": 0,
                "maxlength": 10000,
                "optional": True,
            },
            "postdata_raw": {
                "type": str,
                "minlength": 0,
                "maxlength": 10000,
                "optional": True,
            },
            "check_status_code": {
                "type": int,
                "default": 200
            },
            "check_content": {
                "type": str,
                "maxlength": 100,
                "default": ".*"
            }
        })
        return self._v_http

    def _init_script_params(self):
        self._v_script = Validator()
        self._v_script.set(self._default_params())
        self._v_script.set({
            "message_template": {
                "type": str,
                "minlength": 0,
                "maxlength": 20000,
                "default": "\n".join([
                    "*** Status for host %hostname% (%ipaddr%) at %date% ***",
                    "",
                    "https://%webgui_domain%/#hosts/%host_id%",
                    "%host_description%",
                    "",
                    "%begin_loop%",
                    "---",
                    "Service ID: %service_id%",
                    "Service: %service_name%",
                    "Status: %service_status%",
                    "Message: %service_message%",
                    "%end_loop%"
                ]),
            },
            "script": {
                "type": str,
                "maxlength": 50,
                "regex": r"^[a-zA-Z_0-9.\-]+\Z"
            },
            "params": {
                "type": dict,
                "maxlength": 20000,
                "allow_empty": True,
                "optional": True
            }
        })
        return self._v_script
